@@ -1,6 +1,6 @@
 import logging
 
-from isc_common.fields.related import ForeignKeyCascade, ForeignKeyProtect
+from isc_common.fields.related import ForeignKeyProtect
 from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
 from kaf_pas.kd.models.documents import Documents
 from kaf_pas.kd.models.uploades import Uploades
