@@ -123,6 +123,7 @@ class Ready_2_launchManager(AuditManager):
     @staticmethod
     def make(demand, user, ready_2_launch=None, props=32):
         from kaf_pas.production.models.ready_2_launch_detail import Ready_2_launch_detail
+        from isc_common.models.deleted_progresses import Deleted_progresses
 
         if isinstance(demand, int):
             demand = Demand.objects.get(id=demand)
@@ -164,127 +165,127 @@ class Ready_2_launchManager(AuditManager):
 
         demand_str = f'<h3>Оценка готовности к запуску: Заказ № {demand.code} от {DateToStr(demand.date)}</h3>'
 
-        with transaction.atomic():
-            for _ in Demand.objects.filter(id=demand.id).select_for_update():
-                with managed_progress(
-                        id=f'demand_{demand.id}_{ready_2_launch.props}_{user.id}',
-                        qty=Item_refs.objects.get_descendants_count(
-                            id=demand.precent_item.item.id,
-                            # distinct='distinct',
-                            where_clause=f'''where parent_id != {p_id}'''
-                        ),
-                        user=user,
-                        message=demand_str,
-                        title='Выполнено',
-                        props=TurnBitOn(0, 0)
+        try:
+            settings.LOCKS.acquire(f'Ready_2_launchManager.make_{demand.id}')
+            with transaction.atomic():
+                for _ in Demand.objects.filter(id=demand.id).select_for_update():
+                    with managed_progress(
+                            id=f'demand_{demand.id}_{ready_2_launch.props}_{user.id}',
+                            qty=Item_refs.objects.get_descendants_count(
+                                id=demand.precent_item.item.id,
+                                # distinct='distinct',
+                                where_clause=f'''where parent_id != {p_id}'''
+                            ),
+                            user=user,
+                            message=demand_str,
+                            title='Выполнено',
+                            props=TurnBitOn(0, 0)
 
-                ) as progress:
+                    ) as progress:
 
-                    if not ready_2_launch:
-                        ready_2_launch, _ = Ready_2_launch.objects.get_or_create(demand=demand)
+                        if not ready_2_launch:
+                            ready_2_launch, _ = Ready_2_launch.objects.get_or_create(demand=demand)
 
-                    items_refs_stack = Item_refs_Stack()
-                    items_refs_stack.add_parents(demand.precent_item.item.id)
+                        items_refs_stack = Item_refs_Stack()
+                        items_refs_stack.add_parents(demand.precent_item.item.id)
 
-                    for item_ref in Item_refs.objects.get_descendants(
-                            id=demand.precent_item.item.id,
-                            where_clause=f'''where parent_id != {p_id}  and is_bit_on(props::integer, 1) = true''',
-                            # distinct='distinct'
-                    ):
-                        notes = []
+                        for item_ref in Item_refs.objects.get_descendants(
+                                id=demand.precent_item.item.id,
+                                where_clause=f'''where parent_id != {p_id}  and is_bit_on(props::integer, 1) = true''',
+                                # distinct='distinct'
+                        ):
+                            notes = []
 
-                        # items_refs_stack.push(item_ref, lambda stack, item: len([it for it in stack if it.id != item_ref.id]) == 0)
-                        items_refs_stack.push(item_ref)
+                            # items_refs_stack.push(item_ref, lambda stack, item: len([it for it in stack if it.id != item_ref.id]) == 0)
+                            items_refs_stack.push(item_ref)
 
-                        operations_cnt = Operations_item.objects.filter(item=item_ref.child).count()
-                        section = None
-                        item_full_name = None
-                        item_full_name_obj = None
-                        cnt_not1 = 0
+                            operations_cnt = Operations_item.objects.filter(item=item_ref.child).count()
+                            section = None
+                            item_full_name = None
+                            item_full_name_obj = None
+                            cnt_not1 = 0
 
-                        try:
-                            item_line = Item_line.objects.get(parent=item_ref.parent, child=item_ref.child)
-                            section = item_line.section
+                            try:
+                                item_line = Item_line.objects.get(parent=item_ref.parent, child=item_ref.child)
+                                section = item_line.section
 
-                            if section and section != 'Документация':
-                                if IsBitOn(props, 6) and not item_line.qty:
-                                    cnt_not1 += 1
-                                    notes.append(f'Не указано количество.')
-                                    if not item_full_name:
-                                        item_full_name_obj = items_refs_stack.get_full_path_obj
-                                        item_full_name = items_refs_stack.get_full_path
+                                if section and section != 'Документация':
+                                    if IsBitOn(props, 6) and not item_line.qty:
+                                        cnt_not1 += 1
+                                        notes.append(f'Не указано количество.')
+                                        if not item_full_name:
+                                            item_full_name_obj = items_refs_stack.get_full_path_obj
+                                            item_full_name = items_refs_stack.get_full_path
 
-                        except Item_line.DoesNotExist:
-                            cnt_not += 1
-                            notes.append(f'Не входит в детализацию.')
-                            if not item_full_name:
-                                item_full_name_obj = items_refs_stack.get_full_path_obj
-                                item_full_name = items_refs_stack.get_full_path
-
-                        if section and section != 'Документация':
-                            if operations_cnt == 0:
-                                if IsBitOn(props, 5):
-                                    cnt_not += 1
-                                    notes.append(f'Не указаны операции.')
+                            except Item_line.DoesNotExist:
+                                cnt_not += 1
+                                notes.append(f'Не входит в детализацию.')
+                                if not item_full_name:
                                     item_full_name_obj = items_refs_stack.get_full_path_obj
                                     item_full_name = items_refs_stack.get_full_path
-                            else:
-                                for operation in Operations_item.objects.filter(item=item_ref.child):
-                                    if IsBitOn(props, 0) and not operation.qty:
-                                        cnt_not1 = 1
-                                        notes.append(f'Операция: {operation.operation.full_name} не указана длительность.')
-                                        if not item_full_name:
-                                            item_full_name_obj = items_refs_stack.get_full_path_obj
-                                            item_full_name = items_refs_stack.get_full_path
 
-                                    if IsBitOn(props, 1) and not operation.num:
-                                        cnt_not1 = 1
-                                        notes.append(f'Операция: {operation.operation.full_name} не указан № п/п.')
-                                        if not item_full_name:
-                                            item_full_name_obj = items_refs_stack.get_full_path_obj
-                                            item_full_name = items_refs_stack.get_full_path
-
-                                    if IsBitOn(props, 2):
-                                        operation_material_cnt = Operation_material.objects.filter(operationitem=operation).count()
-                                        if operation_material_cnt == 0:
+                            if section and section != 'Документация':
+                                if operations_cnt == 0:
+                                    if IsBitOn(props, 5):
+                                        cnt_not += 1
+                                        notes.append(f'Не указаны операции.')
+                                        item_full_name_obj = items_refs_stack.get_full_path_obj
+                                        item_full_name = items_refs_stack.get_full_path
+                                else:
+                                    for operation in Operations_item.objects.filter(item=item_ref.child):
+                                        if IsBitOn(props, 0) and not operation.qty:
                                             cnt_not1 = 1
-                                            notes.append(f'Операция: {operation.operation.full_name} не указаны материалы или стандартные изделия.')
+                                            notes.append(f'Операция: {operation.operation.full_name} не указана длительность.')
                                             if not item_full_name:
                                                 item_full_name_obj = items_refs_stack.get_full_path_obj
                                                 item_full_name = items_refs_stack.get_full_path
 
-                                    if IsBitOn(props, 3):
-                                        operation_resources_cnt = Operation_resources.objects.filter(operationitem=operation).count()
-                                        if operation_resources_cnt == 0:
+                                        if IsBitOn(props, 1) and not operation.num:
                                             cnt_not1 = 1
-                                            notes.append(f'Операция: {operation.operation.full_name} не указан ресурс либо место выполнения.')
+                                            notes.append(f'Операция: {operation.operation.full_name} не указан № п/п.')
                                             if not item_full_name:
                                                 item_full_name_obj = items_refs_stack.get_full_path_obj
                                                 item_full_name = items_refs_stack.get_full_path
 
-                                    if IsBitOn(props, 4) and not operation.ed_izm:
-                                        cnt_not1 = 1
-                                        notes.append(f'Операция: {operation.operation.full_name} не указана единица измерения.')
-                                        if not item_full_name:
-                                            item_full_name_obj = items_refs_stack.get_full_path_obj
-                                            item_full_name = items_refs_stack.get_full_path
+                                        if IsBitOn(props, 2):
+                                            operation_material_cnt = Operation_material.objects.filter(operationitem=operation).count()
+                                            if operation_material_cnt == 0:
+                                                cnt_not1 = 1
+                                                notes.append(f'Операция: {operation.operation.full_name} не указаны материалы или стандартные изделия.')
+                                                if not item_full_name:
+                                                    item_full_name_obj = items_refs_stack.get_full_path_obj
+                                                    item_full_name = items_refs_stack.get_full_path
 
+                                        if IsBitOn(props, 3):
+                                            operation_resources_cnt = Operation_resources.objects.filter(operationitem=operation).count()
+                                            if operation_resources_cnt == 0:
+                                                cnt_not1 = 1
+                                                notes.append(f'Операция: {operation.operation.full_name} не указан ресурс либо место выполнения.')
+                                                if not item_full_name:
+                                                    item_full_name_obj = items_refs_stack.get_full_path_obj
+                                                    item_full_name = items_refs_stack.get_full_path
 
-                        if len(notes) > 0:
-                            notes_str = "\n".join(notes)
-                            Ready_2_launch_detail.objects.get_or_create(
-                                ready=ready_2_launch,
-                                notes=notes_str,
-                                item_full_name=item_full_name,
-                                item_full_name_obj=item_full_name_obj,
-                            )
-                        cnt_not += cnt_not1
-                        step_res = progress.step()
-                        cnt += 1
-                        if step_res != 0:
-                            break
+                                        if IsBitOn(props, 4) and not operation.ed_izm:
+                                            cnt_not1 = 1
+                                            notes.append(f'Операция: {operation.operation.full_name} не указана единица измерения.')
+                                            if not item_full_name:
+                                                item_full_name_obj = items_refs_stack.get_full_path_obj
+                                                item_full_name = items_refs_stack.get_full_path
+
+                            if len(notes) > 0:
+                                notes_str = "\n".join(notes)
+                                Ready_2_launch_detail.objects.get_or_create(
+                                    ready=ready_2_launch,
+                                    notes=notes_str,
+                                    item_full_name=item_full_name,
+                                    item_full_name_obj=item_full_name_obj,
+                                )
+                            cnt_not += cnt_not1
+                            step_res = progress.step()
+                            cnt += 1
+                            if step_res != 0:
+                                raise ProgressDroped(progress_deleted)
 
-                    if step_res == 0:
                         ready = round(100 - cnt_not / cnt * 100, 2)
                         all_notes.append(f'{ready}%')
                         notes_str = "\n".join(all_notes)
@@ -293,8 +294,12 @@ class Ready_2_launchManager(AuditManager):
 
                         options_str = "\n" + "\n".join(options)
                         settings.EVENT_STACK.EVENTS_PRODUCTION_READY_2_LAUNCH.send_message(f'Выполнена {demand_str} <h3>готовность: {ready} </h3>{options_str}<p/>')
-                    else:
-                        raise ProgressDroped(progress_deleted)
+            settings.LOCKS.release(f'Ready_2_launchManager.make_{demand.id}')
+        except ProgressDroped as ex:
+            Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
+            settings.LOCKS.release(f'Ready_2_launchManager.make_{demand.id}')
+            raise ex
+
 
     def createFromRequest(self, request):
         request = DSRequest(request=request)
@@ -365,66 +370,6 @@ class Ready_2_launchManager(AuditManager):
         data = request.get_data()
         return data
 
-    def makeTmpMatViewFromRequest(self, request):
-        from kaf_pas.production.models.ready_2_launch_detail import Ready_2_launch_detail
-        from isc_common.models.mat_view import Mat_view
-
-        if not isinstance(request, DSRequest):
-            request = DSRequest(request=request)
-        data = request.get_data()
-
-        ready_2_launch_detail = Ready_2_launch_detail.objects.get(id=data.get('id'))
-        mat_view_name = f'ready_2_launch_detail_{ready_2_launch_detail.id}_mview'
-        try:
-            Mat_view.objects.get(relname=mat_view_name)
-        except Mat_view.DoesNotExist:
-            create_tmp_mat_view(f'''SELECT
-                                           t."isFolder",
-                                           t."STMP_1_id",
-                                           t."STMP_2_id",
-                                           t.deliting,
-                                           t.document__file_document,
-                                           t.document_id,
-                                           t.editing,
-                                           t.id,
-                                           t.lastmodified,
-                                           t.parent_id,
-                                           t.props,
-                                           t.qty_operations,
-                                           t.refs_id,
-                                           t.refs_props,
-                                           t.relevant,
-                                           t.section,
-                                           t.version,
-                                           t.where_from
-                                    FROM production_ready_2_launch_detail s
-                                             CROSS JOIN LATERAL
-                                        json_to_recordset(s.item_full_name_obj::json) as t(
-                                                                                           "deliting" boolean,
-                                                                                           "document__file_document" text,
-                                                                                           "document_id" bigint,
-                                                                                           "editing" boolean,
-                                                                                           "id" bigint,
-                                                                                           "isFolder" boolean,
-                                                                                           "lastmodified" text,
-                                                                                           "parent_id" bigint,
-                                                                                           "props" bigint,
-                                                                                           "qty_operations" int4,
-                                                                                           "refs_id" bigint,
-                                                                                           "refs_props" bigint,
-                                                                                           "relevant" text,
-                                                                                           "section" text,
-                                                                                           "STMP_1_id" bigint,
-                                                                                           "STMP_2_id" bigint,
-                                                                                           "version" int4,
-                                                                                           "where_from" text
-                                                                                            )
-                                        where s.id = {ready_2_launch_detail.id}''', mat_view_name=mat_view_name)
-            logger.debug(f'Created MatView: {mat_view_name}')
-
-        data.setdefault('matViewName', mat_view_name)
-        return data
-
     @staticmethod
     def getRecord(record):
         res = {
